@@ -1,18 +1,11 @@
 ##' Create a dichotomous response model and associated hyperparameters.
 ##'
-##' This function instantiates a dichotomous response model. The
-##' discrimination prior defaults to the lognormal distribution with
-##' \code{meanlog=0} and \code{sdlog=.5}. The guessing prior is the
-##' beta distribution. See the source code for details.  For
-##' discussion on the choice of these Bayesian priors see Baker & Kim
-##' (2004, pp. 187-188).
+##' Bayesian priors are only used to generate plausible random
+##' parameters. For discussion on the choice of these priors see Baker
+##' & Kim (2004, pp. 187-188).
 ##'
-##' It is not yet possible to further customize the Bayesian
-##' priors. The API will change before the 1.0 release.
-##' 
 ##' @param numChoices the number of choices in the question
 ##' @param dimensions the number of dimensions
-##' @param D defaults to 1 or pass in \code{\link{rpf.ogive}}
 ##' @param multidimensional whether to use a multidimensional model.
 ##' Defaults to \code{TRUE} when \code{dimensions>1} and
 ##' \code{FALSE} when \code{dimensions==1}.
@@ -20,7 +13,7 @@
 ##' @export
 ##' @references Baker & Kim (2004). Item Response Theory: Parameter
 ##' Estimation Techniques. Marcel Dekker, Inc.
-rpf.drm <- function(numChoices=5, dimensions=1, D=1, multidimensional) {
+rpf.drm <- function(numChoices=5, dimensions=1, multidimensional) {
   if (missing(multidimensional)) {
     multidimensional <- dimensions > 1
   }
@@ -30,17 +23,15 @@ rpf.drm <- function(numChoices=5, dimensions=1, D=1, multidimensional) {
   guess.weight <- 20
   guessing <- (1/numChoices)
   if (!multidimensional) {
-    new("rpf.1dim.drm", numOutcomes=2, D=D, numParam=3,
+    new("rpf.1dim.drm", numOutcomes=2, numParam=3,
         dimensions=1,
-        guessing=guessing,
         a.prior.meanlog=0,
         a.prior.sdlog=.5,
         c.prior.alpha=guess.weight*guessing+1,
         c.prior.beta=guess.weight*(1-guessing)+1)
   } else {
-    new("rpf.mdim.drm", numOutcomes=2, D=D, dimensions=dimensions,
+    new("rpf.mdim.drm", numOutcomes=2, dimensions=dimensions,
         numParam=2+dimensions,
-        guessing=guessing,
         a.prior.meanlog=0,
         a.prior.sdlog=.5,
         c.prior.alpha=guess.weight*guessing+1,
@@ -50,14 +41,10 @@ rpf.drm <- function(numChoices=5, dimensions=1, D=1, multidimensional) {
 
 ### 1dim
 
-setMethod("rpf.prob", signature(m="rpf.1dim.drm", param="numeric",
+setMethod("rpf.logprob", signature(m="rpf.1dim.drm", param="numeric",
                                 theta="numeric"),
           function(m, param, theta) {
-            a <- param[1] * m@D
-            b <- param[2]
-            c <- param[3]
-            p <- c + (1-c)/(1+exp(-a*(theta-b)))
-            cbind(1-p,p)
+            t(.Call("rpf_1dim_drm_logprob_wrapper", param, theta))
           })
 
 ##' @references
@@ -66,7 +53,7 @@ setMethod("rpf.info", signature(m="rpf.1dim.drm", param="numeric",
                                 theta="numeric"),
           function(m, param, theta) {
             p <- rpf.prob(m, param, theta)
-            a <- param[1] * m@D
+            a <- param[1]
             c <- param[3]
             a^2 * (p[,1]/p[,2]) * ((p[,2]-c)^2/(1-c)^2)
           })
@@ -83,7 +70,7 @@ setMethod("rpf.rparam", signature(m="rpf.1dim.drm"),
 
 setMethod("rpf.startingParam", signature(m="rpf.1dim.drm"),
           function(m) {
-            c(a=1, b=0, c=m@guessing)
+            c(a=1, b=0, c=0)
           })
 
 setMethod("rpf.getLocation", signature(m="rpf.1dim.drm", param="numeric"),
@@ -99,16 +86,10 @@ setMethod("rpf.setLocation", signature(m="rpf.1dim.drm", param="numeric", loc="n
 
 ### mdim
 
-##' @author Jonathan Weeks <weeksjp@@gmail.com>
-setMethod("rpf.prob", signature(m="rpf.mdim.drm", param="numeric",
-                                theta="matrix"),
+setMethod("rpf.logprob", signature(m="rpf.mdim.drm", param="numeric",
+                                   theta="matrix"),
           function(m, param, theta) {
-            a <- param[1:m@dimensions] * m@D
-            p.rest <- param[-1:-m@dimensions]
-            b <- p.rest[1]
-            c <- p.rest[2]
-            p <- c + (1-c)/(1+exp(-(theta %*% a + b)))
-            cbind(1-p,p)
+            t(.Call("rpf_mdim_drm_logprob_wrapper", m@dimensions, param, t(theta)))
           })
 
 setMethod("rpf.rparam", signature(m="rpf.mdim.drm"),
@@ -122,7 +103,7 @@ setMethod("rpf.rparam", signature(m="rpf.mdim.drm"),
 
 setMethod("rpf.startingParam", signature(m="rpf.mdim.drm"),
           function(m) {
-            c(a=rep(1,m@dimensions), b=0, c=m@guessing)
+            c(a=rep(1,m@dimensions), b=0, c=0)
           })
 
 setMethod("rpf.getLocation", signature(m="rpf.mdim.drm", param="numeric"),
